/*
 * MicroTimer.h - simple high-precision timer
 *
 * Copyright (c) 2005-2009 Tobias Doerffel <tobydox/at/users.sourceforge.net>
 *
 * This file is part of LMMS - https://lmms.io
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program (see COPYING); if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA.
 *
 */

#ifndef LMMS_MICRO_TIMER_H
#define LMMS_MICRO_TIMER_H

#include <chrono>

namespace lmms
{

class MicroTimer
{
	using time_point = std::chrono::steady_clock::time_point;

public:
	MicroTimer();
	~MicroTimer() = default;

	void reset();
	int elapsed() const;

private:
	time_point begin;
} ;


} // namespace lmms

#endif // LMMS_MICRO_TIMER_H
