#define LMMS_VERSION "1.3.0-alpha.1.965+1155590"
#define LMMS_PROJECT_COPYRIGHT "2008-2026 LMMS Developers"
#define LMMS_BUILD_OPTIONS "LMMS_HAVE_ALSA='TRUE' LMMS_HAVE_ALSA_ASOUNDLIB_H='1' LMMS_HAVE_AWE_VOICE_H='' LMMS_HAVE_CTYPE_H='1' LMMS_HAVE_FCNTL_H='1' LMMS_HAVE_FLUIDSYNTH='TRUE' LMMS_HAVE_JACK='TRUE' LMMS_HAVE_JACK_PRENAME='1' LMMS_HAVE_LIBRT='1' LMMS_HAVE_LINUX_AWE_VOICE_H='' LMMS_HAVE_LOCALE_H='1' LMMS_HAVE_LV2='TRUE' LMMS_HAVE_MACHINE_SOUNDCARD_H='' LMMS_HAVE_MP3LAME='TRUE' LMMS_HAVE_OGGVORBIS='TRUE' LMMS_HAVE_OSS='TRUE' LMMS_HAVE_PORTAUDIO='TRUE' LMMS_HAVE_PROCESS_H='' LMMS_HAVE_PTHREAD_H='1' LMMS_HAVE_PULSEAUDIO='TRUE' LMMS_HAVE_SCHED_H='1' LMMS_HAVE_SDL='TRUE' LMMS_HAVE_SEMAPHORE_H='1' LMMS_HAVE_SF_COMPLEVEL='1' LMMS_HAVE_SID='TRUE' LMMS_HAVE_SNDFILE_MP3='TRUE' LMMS_HAVE_SOUNDCARD_H='' LMMS_HAVE_STRING_H='1' LMMS_HAVE_SUIL='TRUE' LMMS_HAVE_SYS_ASOUNDLIB_H='1' LMMS_HAVE_SYS_IOCTL_H='1' LMMS_HAVE_SYS_IPC_H='1' LMMS_HAVE_SYS_PRCTL_H='1' LMMS_HAVE_SYS_SOUNDCARD_H='1' LMMS_HAVE_SYS_TIMES_H='1' LMMS_HAVE_SYS_TIME_H='1' LMMS_HAVE_SYS_TYPES_H='1' LMMS_HAVE_UNISTD_H='1' LMMS_HAVE_VST='TRUE' LMMS_HAVE__USR_SRC_SYS_GNU_I386_ISA_SOUND_AWE_VOICE_H='' LMMS_HAVE__USR_SRC_SYS_I386_ISA_SOUND_AWE_VOICE_H='' WANT_ALSA='ON' WANT_CALF='Off' WANT_CAPS='Off' WANT_CARLA='Off' WANT_CMT='Off' WANT_CPACK_TARBALL='OFF' WANT_DEBUG_ASAN='OFF' WANT_DEBUG_CPACK='OFF' WANT_DEBUG_FPE='OFF' WANT_DEBUG_GPROF='OFF' WANT_DEBUG_MSAN='OFF' WANT_DEBUG_TSAN='OFF' WANT_DEBUG_UBSAN='OFF' WANT_GIG='ON' WANT_JACK='ON' WANT_LV2='ON' WANT_MP3LAME='ON' WANT_OGGVORBIS='ON' WANT_OSS='ON' WANT_PORTAUDIO='ON' WANT_PULSEAUDIO='ON' WANT_QT5='OFF' WANT_QT6='ON' WANT_SDL='ON' WANT_SF2='ON' WANT_SID='ON' WANT_SNDIO='ON' WANT_SOUNDIO='ON' WANT_STK='ON' WANT_SUIL='ON' WANT_SWH='Off' WANT_TAP='Off' WANT_VST='ON' WANT_VST_32='ON' WANT_VST_64='ON' WANT_WEAKJACK='OFF' WANT_WINMM='OFF'"
